@@ -39,7 +39,7 @@ using namespace unity;
 namespace scopes_ng {
 
 // FIXME: this should be in a common place
-#define CATEGORY_JSON_DEFAULTS R"({"schema-version":1,"template": {"category-layout":"grid","card-layout":"vertical","card-size":"small","overlay-mode":null,"collapsed-rows":2}, "components": { "title":null, "art": { "aspect-ratio":1.0, "fill-mode":"crop" }, "subtitle":null, "mascot":null, "emblem":null, "summary":null, "attributes":null, "background":null }, "resources":{}})"
+#define CATEGORY_JSON_DEFAULTS R"({"schema-version":1,"template": {"category-layout":"grid","card-layout":"vertical","card-size":"small","overlay-mode":null,"collapsed-rows":2}, "components": { "title":null, "art": { "aspect-ratio":1.0 }, "subtitle":null, "mascot":null, "emblem":null, "summary":null, "attributes": { "max-count":2 }, "background":null, "overlay-color":null }, "resources":{}})"
 
 class CategoryData
 {
@@ -89,6 +89,12 @@ public:
             QString::fromStdString(m_category->icon()) : m_catIcon;
     }
 
+    QString headerLink() const
+    {
+        return m_category && m_category->query() ?
+            QString::fromStdString(m_category->query()->to_uri()) : QString();
+    }
+
     std::string rawTemplate() const
     {
         return m_rawTemplate;
@@ -134,6 +140,15 @@ public:
         return result;
     }
 
+    int getMaxAttributes() const
+    {
+        QJsonObject components_obj = m_components.toObject();
+        QJsonObject attrs_obj = components_obj.value("attributes").toObject();
+        QJsonValue max_count_val = attrs_obj.value("max-count");
+
+        return max_count_val.toInt(2);
+    }
+
     QVector<int> updateAttributes(scopes::Category::SCPtr category)
     {
         QVector<int> roles;
@@ -144,6 +159,17 @@ public:
         if (category->icon() != m_category->icon()) {
             roles.append(Categories::RoleIcon);
         }
+        std::string oldQuery;
+        std::string newQuery;
+        if (m_category->query()) {
+            oldQuery = m_category->query()->to_uri();
+        }
+        if (category->query()) {
+            newQuery = category->query()->to_uri();
+        }
+        if (oldQuery != newQuery) {
+            roles.append(Categories::RoleHeaderLink);
+        }
         if (category->renderer_template().data() != m_rawTemplate) {
             roles.append(Categories::RoleRawRendererTemplate);
 
@@ -193,19 +219,6 @@ public:
         return m_isSpecial;
     }
 
-private:
-    static QJsonValue* DEFAULTS;
-    scopes::Category::SCPtr m_category;
-    QString m_catId;
-    QString m_catTitle;
-    QString m_catIcon;
-    std::string m_rawTemplate;
-    QJsonValue m_rendererTemplate;
-    QJsonValue m_components;
-    ResultsModel* m_resultsModel;
-    QPointer<QObject> m_countObject;
-    bool m_isSpecial;
-
     static bool parseTemplate(std::string const& raw_template, QJsonValue* renderer, QJsonValue* components)
     {
         // lazy init of the defaults
@@ -239,6 +252,19 @@ private:
         return true;
     }
 
+private:
+    static QJsonValue* DEFAULTS;
+    scopes::Category::SCPtr m_category;
+    QString m_catId;
+    QString m_catTitle;
+    QString m_catIcon;
+    std::string m_rawTemplate;
+    QJsonValue m_rendererTemplate;
+    QJsonValue m_components;
+    ResultsModel* m_resultsModel;
+    QPointer<QObject> m_countObject;
+    bool m_isSpecial;
+
     static QJsonValue mergeOverrides(QJsonValue const& defaultVal, QJsonValue const& overrideVal)
     {
         if (overrideVal.isObject() && defaultVal.isObject()) {
@@ -276,23 +302,8 @@ private:
 QJsonValue* CategoryData::DEFAULTS = nullptr;
 
 Categories::Categories(QObject* parent)
-    : QAbstractListModel(parent)
+    : unity::shell::scopes::CategoriesInterface(parent)
 {
-    m_roles[Categories::RoleCategoryId] = "categoryId";
-    m_roles[Categories::RoleName] = "name";
-    m_roles[Categories::RoleIcon] = "icon";
-    m_roles[Categories::RoleRawRendererTemplate] = "rawRendererTemplate";
-    m_roles[Categories::RoleRenderer] = "renderer";
-    m_roles[Categories::RoleComponents] = "components";
-    m_roles[Categories::RoleProgressSource] = "progressSource";
-    m_roles[Categories::RoleResults] = "results";
-    m_roles[Categories::RoleCount] = "count";
-}
-
-QHash<int, QByteArray>
-Categories::roleNames() const
-{
-    return m_roles;
 }
 
 int Categories::rowCount(const QModelIndex& parent) const
@@ -353,6 +364,7 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
                 resultsModel = catData->resultsModel();
                 if (resultsModel) {
                     resultsModel->setComponentsMapping(catData->getComponentsMapping());
+                    resultsModel->setMaxAtrributesCount(catData->getMaxAttributes());
                 }
             }
             beginInsertRows(QModelIndex(), emptyIndex, emptyIndex);
@@ -367,6 +379,7 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
                 resultsModel = catData->resultsModel();
                 if (resultsModel) {
                     resultsModel->setComponentsMapping(catData->getComponentsMapping());
+                    resultsModel->setMaxAtrributesCount(catData->getMaxAttributes());
                 }
                 QModelIndex changedIndex(this->index(index));
                 dataChanged(changedIndex, changedIndex, changedRoles);
@@ -384,6 +397,7 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
         m_categories.insert(emptyIndex, QSharedPointer<CategoryData>(catData));
         resultsModel->setCategoryId(QString::fromStdString(category->id()));
         resultsModel->setComponentsMapping(catData->getComponentsMapping());
+        resultsModel->setMaxAtrributesCount(catData->getMaxAttributes());
         m_categoryResults[category->id()] = resultsModel;
 
         endInsertRows();
@@ -426,6 +440,12 @@ void Categories::clearAll()
     dataChanged(changeStart, changeEnd, roles);
 }
 
+
+bool Categories::parseTemplate(std::string const& raw_template, QJsonValue* renderer, QJsonValue* components)
+{
+    return CategoryData::parseTemplate(raw_template, renderer, components);
+}
+
 bool Categories::overrideCategoryJson(QString const& categoryId, QString const& json)
 {
     int idx = getCategoryIndex(categoryId);
@@ -436,6 +456,7 @@ bool Categories::overrideCategoryJson(QString const& categoryId, QString const&
         }
         if (catData->resultsModel()) {
             catData->resultsModel()->setComponentsMapping(catData->getComponentsMapping());
+            catData->resultsModel()->setMaxAtrributesCount(catData->getMaxAttributes());
         }
         QModelIndex changeIndex(index(idx));
         QVector<int> roles;
@@ -505,8 +526,8 @@ Categories::data(const QModelIndex& index, int role) const
             return catData->rendererTemplate().toVariant();
         case RoleComponents:
             return catData->components().toVariant();
-        case RoleProgressSource:
-            return QVariant();
+        case RoleHeaderLink:
+            return catData->headerLink();
         case RoleResults:
             return QVariant::fromValue(resultsModel);
         case RoleCount:
