@@ -31,8 +31,10 @@ using namespace unity::scopes;
 class MyQuery : public SearchQueryBase
 {
 public:
-    MyQuery(string const& query) :
-        query_(query)
+    MyQuery(CannedQuery const& query, SearchMetadata const& metadata) :
+        SearchQueryBase(query, metadata),
+        query_(query.query_string()),
+        department_id_(query.department_id())
     {
     }
 
@@ -75,7 +77,7 @@ public:
         }
         else if (query_ == "attributes")
         {
-            CategoryRenderer rating_rndr(R"({"schema-version": 1, "components": {"title": "title", "attributes": "attributes"}})");
+            CategoryRenderer rating_rndr(R"({"schema-version": 1, "components": {"title": "title", "attributes": {"field": "attributes", "max-count":3}}})");
             auto cat = reply->register_category("cat1", "Category 1", "", rating_rndr);
             CategorisedResult res(cat);
             res.set_uri("test:uri");
@@ -134,14 +136,33 @@ public:
             res.set_title("result for: \"" + query_ + "\"");
             reply->push(res);
         }
+        else if (query_ == "query")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
+            CategorisedResult res(cat);
+            res.set_uri("test:query");
+            res.set_title("result for: \"" + query_ + "\"");
+            reply->push(res);
+        }
+        else if (query_ == "expandable-widget")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
+            CategorisedResult res(cat);
+            res.set_uri("test:expandable-widget");
+            res.set_art("art");
+            res.set_title("result for: \"" + query_ + "\"");
+            reply->push(res);
+        }
         else if (query_ == "perform-query")
         {
             CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
             auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
             CategorisedResult res(cat);
-            res.set_uri("test:perform-query");
+            res.set_uri("scope://test:perform-query"); // FIXME: workaround for https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1428063
             res.set_title("result for: \"" + query_ + "\"");
-            res["scope-id"] = "mock-scope";
+            res["scope-id"] = "mock-scope-ttl";
             res.set_intercept_activation();
             reply->push(res);
         }
@@ -150,12 +171,31 @@ public:
             CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
             auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
             CategorisedResult res(cat);
-            res.set_uri("test:perform-query");
+            res.set_uri("scope://test:perform-query"); // FIXME: workaround for https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1428063
             res.set_title("result for: \"" + query_ + "\"");
             res["scope-id"] = "nonexisting-scope";
             res.set_intercept_activation();
             reply->push(res);
         }
+        else if (query_ == "scope-uri")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
+            CategorisedResult res(cat);
+            res.set_uri("scope://mock-scope?q=next-scope-query");
+            res.set_title("result for: \"" + query_ + "\"");
+            reply->push(res);
+        }
+        else if (query_ == "next-scope-query")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title", "art": "art"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
+            CategorisedResult res(cat);
+            res.set_uri("next-scope-query");
+            res.set_art("next-scope-query-art");
+            res.set_title("result for: \"" + query_ + "\"");
+            reply->push(res);
+        }
         else if (query_ == "two-categories")
         {
             auto cat1 = reply->register_category("cat1", "Category 1", "");
@@ -193,6 +233,17 @@ public:
             res1.set_title("result for: \"" + query_ + "\"");
             reply->push(res1);
         }
+        else if (query_ == "expansion-query")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", query(), minimal_rndr);
+            CategorisedResult res(cat);
+            res.set_uri("test:uri");
+            res.set_title("result for: \"" + query_ + "\"");
+            res.set_art("art");
+            res.set_dnd_uri("test:dnd_uri");
+            reply->push(res);
+        }
         else
         {
             auto cat = reply->register_category("cat1", "Category 1", "");
@@ -201,6 +252,9 @@ public:
             res.set_title("result for: \"" + query_ + "\"");
             res.set_art("art");
             res.set_dnd_uri("test:dnd_uri");
+            res["session-id"] = search_metadata()["session-id"].get_string();
+            res["query-id"] = Variant(search_metadata()["query-id"].get_int());
+            res["booleanness"] = Variant(true);
             res.set_intercept_activation();
             reply->push(res);
         }
@@ -208,13 +262,15 @@ public:
 
 private:
     string query_;
+    string department_id_;
 };
 
 class MyPreview : public PreviewQueryBase
 {
 public:
-    MyPreview(Result const& result, Variant const& scope_data = Variant()) :
-        result_(result), scope_data_(scope_data)
+    MyPreview(Result const& result, ActionMetadata const& metadata) :
+        PreviewQueryBase(result, metadata),
+        scope_data_(metadata.scope_data())
     {
     }
 
@@ -228,7 +284,7 @@ public:
 
     virtual void run(PreviewReplyProxy const& reply) override
     {
-        if (result_.uri().find("layout") != std::string::npos)
+        if (result().uri().find("layout") != std::string::npos)
         {
             PreviewWidget w1("img", "image");
             w1.add_attribute_value("source", Variant("foo.png"));
@@ -270,32 +326,79 @@ public:
             reply->push(widgets);
             return;
         }
+        else if (result().uri().find("query") != std::string::npos)
+        {
+            PreviewWidget w1("actions", "actions");
+
+            VariantBuilder builder;
+            auto uri = CannedQuery("mock-scope").to_uri();
+            builder.add_tuple({
+                {"id", Variant("nothing")},
+                {"label", Variant("Do nothing")}
+            });
+            builder.add_tuple({
+                {"id", Variant("query")},
+                {"label", Variant("Search")},
+                {"uri", Variant(uri)}
+            });
+            w1.add_attribute_value("actions", builder.end());
+
+            ColumnLayout l1(1);
+            l1.add_column({"actions"});
+
+            reply->register_layout({l1});
+            PreviewWidgetList widgets({w1});
+            reply->push(widgets);
+            return;
+        }
+        else if (result().uri().find("expandable-widget") != std::string::npos)
+        {
+            PreviewWidget w1("exp", "expandable");
+            w1.add_attribute_value("title", Variant("Expandable widget"));
+            PreviewWidget w2("txt", "text");
+            w2.add_attribute_value("title", Variant("Subwidget"));
+            w2.add_attribute_value("text", Variant("Lorum ipsum"));
+            PreviewWidget w3("img", "image");
+            w3.add_attribute_mapping("source", "src");
+            w1.add_widget(w2);
+            w1.add_widget(w3);
+
+            PreviewWidget w4("img", "image");
+            w4.add_attribute_value("source", Variant("foo.png"));
+
+            PreviewWidgetList widgets({w1, w4});
+            reply->push(widgets);
+            reply->push("src", Variant("bar.png"));
+            return;
+        }
 
         PreviewWidgetList widgets;
-        PreviewWidget w1(R"({"id": "hdr", "type": "header", "components": {"title": "title", "subtitle": "uri", "attribute-1": "extra-data"}})");
+        PreviewWidget w1(R"({"id": "hdr", "type": "header", "components": {"title": "title", "subtitle": "uri", "attribute-1": "extra-data", "session-id": "session-id-val"}})");
         PreviewWidget w2(R"({"id": "img", "type": "image", "components": {"source": "art"}, "zoomable": false})");
         widgets.push_back(w1);
         widgets.push_back(w2);
         reply->push(widgets);
 
         reply->push("extra-data", Variant("foo"));
+        reply->push("session-id-val", Variant(action_metadata()["session-id"]));
     }
 
 private:
-    Result result_;
     Variant scope_data_;
 };
 
 class MyActivation : public ActivationQueryBase
 {
 public:
-    MyActivation(Result const& result) :
-        result_(result), status_(ActivationResponse::HideDash)
+    MyActivation(Result const& result, ActionMetadata const& metadata) :
+        ActivationQueryBase(result, metadata),
+        status_(ActivationResponse::HideDash)
     {
     }
 
-    MyActivation(Result const& result, ActivationResponse::Status status) :
-        result_(result), status_(status)
+    MyActivation(Result const& result, ActionMetadata const& metadata, ActivationResponse::Status status) :
+        ActivationQueryBase(result, metadata),
+        status_(status)
     {
     }
 
@@ -315,7 +418,7 @@ public:
     virtual ActivationResponse activate() override
     {
         if (status_ == ActivationResponse::Status::PerformQuery) {
-            auto resp = ActivationResponse(CannedQuery(result_["scope-id"].get_string()));
+            auto resp = ActivationResponse(CannedQuery(result()["scope-id"].get_string()));
             return resp;
         } else {
             auto resp = ActivationResponse(status_);
@@ -326,30 +429,22 @@ public:
 
 private:
     Variant extra_data_;
-    Result result_;
     ActivationResponse::Status status_;
 };
 
 class MyScope : public ScopeBase
 {
 public:
-    virtual int start(string const&, RegistryProxy const&) override
+    virtual SearchQueryBase::UPtr search(CannedQuery const& q, SearchMetadata const& metadata) override
     {
-        return VERSION;
-    }
-
-    virtual void stop() override {}
-
-    virtual SearchQueryBase::UPtr search(CannedQuery const& q, SearchMetadata const&) override
-    {
-        SearchQueryBase::UPtr query(new MyQuery(q.query_string()));
+        SearchQueryBase::UPtr query(new MyQuery(q, metadata));
         cout << "scope-A: created query: \"" << q.query_string() << "\"" << endl;
         return query;
     }
 
     virtual PreviewQueryBase::UPtr preview(Result const& result, ActionMetadata const& metadata) override
     {
-        PreviewQueryBase::UPtr query(new MyPreview(result, metadata.scope_data()));
+        PreviewQueryBase::UPtr query(new MyPreview(result, metadata));
         cout << "scope-A: created preview query: \"" << result.uri() << "\"" << endl;
         return query;
     }
@@ -358,23 +453,23 @@ public:
     {
         if (widget_id == "actions" && action_id == "hide")
         {
-            return ActivationQueryBase::UPtr(new MyActivation(result));
+            return ActivationQueryBase::UPtr(new MyActivation(result, meta));
         }
         else if (widget_id == "actions" && action_id == "download")
         {
-            MyActivation* response = new MyActivation(result, ActivationResponse::ShowPreview);
+            MyActivation* response = new MyActivation(result, meta, ActivationResponse::ShowPreview);
             response->setExtraData(meta.scope_data());
             return ActivationQueryBase::UPtr(response);
         }
-        return ActivationQueryBase::UPtr(new MyActivation(result, ActivationResponse::NotHandled));
+        return ActivationQueryBase::UPtr(new MyActivation(result, meta, ActivationResponse::NotHandled));
     }
 
-    virtual ActivationQueryBase::UPtr activate(Result const& result, ActionMetadata const&) override
+    virtual ActivationQueryBase::UPtr activate(Result const& result, ActionMetadata const& meta) override
     {
         if (result.uri().find("perform-query") != std::string::npos) {
-            return ActivationQueryBase::UPtr(new MyActivation(result, ActivationResponse::PerformQuery));
+            return ActivationQueryBase::UPtr(new MyActivation(result, meta, ActivationResponse::PerformQuery));
         }
-        return ActivationQueryBase::UPtr(new MyActivation(result));
+        return ActivationQueryBase::UPtr(new MyActivation(result, meta));
     }
 };
 
